<?xml version='1.0'?>														 
<!-- 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   - 
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   - 
   - The Original Code is XML Schema Compiler (Pass1).
   - 
   - The Initial Developer of the Original Code is AEA Technology - Engineering Software.
   -  Portions created by AEA Technology - Engineering Software are
   - Copyright (C) 2000 AEA Technology - Engineering Software.  All
   - Rights Reserved.
   - 
   - Contributor(s): Curt Arnold <curta@hyprotech.com>
  --> 

<xsl:transform xmlns:xsl='http://www.w3.org/1999/XSL/Transform' 
     version="1.0"
	 xmlns:xsc="http://XSDComp.sourceforge.net/Namespaces/2000/XMLSchema-compiled">

<xsl:output method="xml" version="1.0" indent="yes"/>

<xsl:key name="named-simple-types" match="/xsc:package/xsc:schema/xsc:simpleType" use="concat(concat(parent::xsc:schema/@targetNamespace,' '),@name)"/>
<xsl:key name="named-complex-types" match="/xsc:package/xsc:schema/xsc:complexType" use="concat(concat(parent::xsc:schema/@targetNamespace,' '),@name)"/>
<xsl:key name="named-types" match="/xsc:package/xsc:schema/*[name()='xsc:simpleType' or name()='xsc:complexType']" use="concat(concat(parent::xsc:schema/@targetNamespace,' '),@name)"/>
<xsl:key name="named-elements" match="/xsc:package/xsc:schema/xsc:element" use="concat(concat(parent::xsc:schema/@targetNamespace,' '),@name)"/>
<xsl:key name="named-groups" match="/xsc:package/xsc:schema/xsc:group" use="concat(concat(parent::xsc:schema/@targetNamespace,' '),@name)"/>
<xsl:key name="named-attr-groups" match="/xsc:package/xsc:schema/xsc:attributeGroup" use="concat(concat(parent::xsc:schema/@targetNamespace,' '),@name)"/>
<xsl:key name="named-attributes" match="/xsc:package/xsc:schema/xsc:attribute" use="concat(concat(parent::xsc:schema/@targetNamespace,' '),@name)"/>

<xsl:key name="namespaceURI" match="//xsc:namespace" use="@uri"/>

<!-- matches document root and invokes templates for all child nodes -->
<xsl:template match="/">
	<xsl:apply-templates/>
</xsl:template>

<!--  unrecognized elements and attributes get dropped  -->
<xsl:template match="*">
	<xsl:element name="{name()}">
		<xsl:apply-templates select="@*|text()|*"/>
	</xsl:element>
</xsl:template>

<xsl:template match="@*">
	<xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
</xsl:template>

<xsl:template match="text()">
	<xsl:value-of select="text()"/>
</xsl:template>

<xsl:template match="xsc:package">
	<xsc:package>
		<!--  generate xsc:namespace entries for each declared namespace -->
		<xsl:variable name="declaredNamespaces" select="descendant::xsc:namespace"/>
		<xsl:apply-templates select="$declaredNamespaces"/>

		<!--   generate xsc:schema entries for each schema   -->
		<xsl:apply-templates/>
	</xsc:package>
</xsl:template>

<xsl:template match="xsc:prefix">
	<xsc:prefix><xsl:value-of select="text()"/></xsc:prefix>
</xsl:template>

<xsl:template match="xsc:namespace">
	<xsl:variable name="uri" select="@uri"/>
	<!--   my grouping code is not working as I expected   -->
	<xsl:if test="not(preceding::xsc:namespace[@uri=$uri])">
		<xsc:namespace uri="{@uri}" id="{generate-id(.)}">
			<xsl:choose>
				<xsl:when test="@version">
					<xsl:attribute name="version"><xsl:value-of select="@version"/></xsl:attribute>
				</xsl:when>

				<xsl:otherwise>
					<xsl:variable name="versions" select="following::xsc:namespace[@uri = $uri and @version]"/>
					<xsl:if test="$versions[position()=1]">
						<xsl:attribute name="version"><xsl:value-of select="$versions[position()=1]"/></xsl:attribute>
					</xsl:if>
				</xsl:otherwise>
			</xsl:choose>
		</xsc:namespace>
	</xsl:if>
</xsl:template>

<xsl:template match="xsc:schema">
	<xsl:variable name="namespaceID" select="generate-id(key('namespaceURI',xsc:namespace/@uri))"/>
	<xsc:schema id="{generate-id(.)}" namespace="{$namespaceID}">
		<!--  gather together all top-level annotation elements   -->
		<xsl:variable name="annotations" select="xsc:annotation"/>
		<xsl:if test="$annotations">
			<xsc:annotation>
				<xsl:apply-templates select="$annotations/*"/>
			</xsc:annotation>
		</xsl:if>
		<xsl:apply-templates select="xsc:notation"/>

		<!--  all simple types are collected and placed at the head of the 
		          package.  Later passes may assemble them into a tree-inheritance structure
				      (part of the reasoning behind putting an explicit namespace reference in the simpleType) -->
		<xsl:apply-templates select="xsc:simpleType">
			<xsl:with-param name="namespaceID" select="$namespaceID"/>
		</xsl:apply-templates>

		<!--  expand all anonymous simple types.  
              Well not quite all.  If a simpleType appears in a restriction		
              and does not have an explicit base attribute, then it is really
              a bag of facets that need to be interpreted in the context
              of its attributes (or elements) inherited type, which we don't know
              on this pass.		
		-->
		<xsl:apply-templates select="*/descendant::xsc:simpleType[@base or not(parent::*/@derivedBy='restriction')]"/>

		<!--  there should not be unqualified attributes at the top level  -->
		<xsl:if test="xsc:attribute[@form!='qualified']">
			<!--  should have been caught in preprocessor   -->
			<xsl:message>Warning: Unqualified attribute <xsl:value-of select="@name"/> appears as immediate child of schema.  Should have been caught in preprocessor.</xsl:message>
		</xsl:if>

		<xsl:apply-templates select="descendant::xsc:attribute">
			<xsl:with-param name="namespaceID" select="$namespaceID"/>
		</xsl:apply-templates>


		<!--  create attributeUseGroups for all attribute groups and complexTypes   -->
		<xsl:apply-templates select="xsc:attributeGroup|descendant::xsc:complexType" mode="attribute-uses">
			<xsl:with-param name="namespaceID" select="$namespaceID"/>
		</xsl:apply-templates>
		
		<!--  expand all named elements            -->
		<xsl:apply-templates select="xsc:element[@name]">
			<xsl:with-param name="namespaceID" select="$namespaceID"/>
		</xsl:apply-templates>

		<!--  expand all locally scoped elements            -->
		<xsl:apply-templates select="*/descendant::xsc:element[@name]"/>

		<!--  expand all named groups    -->
		<xsl:apply-templates select="xsc:group"/>

		<!--  expand all groups implied in complex type definitions   -->
		<xsl:apply-templates select="descendant::xsc:complexType" mode="element-use"/>

		<!--  expand all named complex types     -->
		<xsl:apply-templates select="xsc:complexType"/>
		<!--  expand all anonymous complex types   -->
		<xsl:apply-templates select="*/descendant::xsc:complexType"/>
    </xsc:schema>
</xsl:template>

<xsl:template match="*" mode="element-use">
	<xsl:message terminate="yes">Fatal Error: Internal error, element tag <xsl:value-of select="name()"/>, this template should not get executed.</xsl:message>
</xsl:template>

<xsl:template match="xsc:complexType" mode="element-use">
	<xsc:group id="{generate-id(.)}_group" compositor="sequence">

		<!--  if there was a base type, then include a reference or restriction to its group    -->
		<xsl:if test="@base">

			<!-- see if there are any elements, groups or anys in a restriction clause   -->
			<xsl:variable name="element-restrictions" select="xsc:restriction/*[name()='xsc:element' or name()='xsc:group' or name()='xsc:any']"/>

			<xsl:choose>
				<!--  if there is a restriction to our base with some element restrictions   -->
				<xsl:when test="$element-restrictions">
					<xsc:restrictedGroup>
						<xsl:call-template name="produce-idref-complex">
							<xsl:with-param name="name" select="@base"/>
							<xsl:with-param name="keyname" select="named-complex-types"/>
							<xsl:with-param name="attname">base</xsl:with-param>
							<xsl:with-param name="suffix">_group</xsl:with-param>
						</xsl:call-template>
						<xsl:apply-templates select="$element-restrictions" mode="element-use"/>
					</xsc:restrictedGroup>
				</xsl:when>

				<!--  if not, emit a groupUse to the base group    -->
				<xsl:otherwise>
					<xsc:groupUse minOccurs="1" maxOccurs="1" unbounded="false">
						<xsl:call-template name="produce-idref-complex">
							<xsl:with-param name="name" select="@base"/>
							<xsl:with-param name="keyname" select="named-complex-types"/>
							<xsl:with-param name="attname">ref</xsl:with-param>
							<xsl:with-param name="suffix">_group</xsl:with-param>
						</xsl:call-template>
					</xsc:groupUse>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>			

		<!--  expand any element related content in the complex type   -->
		<xsl:apply-templates select="xsc:element|xsc:group|xsc:any" mode="element-use"/>

	</xsc:group>
</xsl:template>


<xsl:template match="xsc:any" mode="element-use">
	<xsc:any minOccurs="{@minOccurs}" maxOccurs="{@maxOccurs}" unbounded="{@unbounded}">
		<xsl:apply-templates select="xsc:annotation"/>
		<xsl:if test="xsc:namespaceNor">
			<xsc:namespaceNor>
				<xsl:apply-templates select="xsc:namespaceNor/*" mode="namespace-ref"/>
			</xsc:namespaceNor>
		</xsl:if>
		<xsl:apply-templates select="xsc:namespace" mode="namespace-ref"/>
	</xsc:any>
</xsl:template>


<xsl:template match="xsc:element" mode="element-use">
	<xsc:elementUse minOccurs="{@minOccurs}" maxOccurs="{@maxOccurs}" unbounded="{@unbounded}">
		<xsl:choose>
			<!--  locally scoped element, definition has preceeded us with a generated id  -->
			<xsl:when test="@name">
				<xsl:attribute name="element"><xsl:value-of select="generate-id(.)"/></xsl:attribute>
			</xsl:when>

			<xsl:when test="@ref">
				<xsl:call-template name="produce-idref-element">
					<xsl:with-param name="name" select="@ref"/>
					<xsl:with-param name="attname">element</xsl:with-param>
				</xsl:call-template>
			</xsl:when>

			<xsl:otherwise>
				<xsl:message terminate="yes">Fatal Error: neither name or ref attribute on element.</xsl:message>
			</xsl:otherwise>
		</xsl:choose>
	</xsc:elementUse>
</xsl:template>


<xsl:template match="xsc:group[not(@name or @ref)]" mode="element-use">
	<xsc:groupUse minOccurs="{@minOccurs}" maxOccurs="{@maxOccurs}" unbounded="{@unbounded}">
		<xsc:group compositor="{@compositor}">
			<xsl:apply-templates select="xsc:element|xsc:group|xsc:any" mode="element-use"/>
		</xsc:group>
	</xsc:groupUse>
</xsl:template>

<xsl:template match="xsc:group[@ref]" mode="element-use">
	<xsc:groupUse  minOccurs="{@minOccurs}" maxOccurs="{@maxOccurs}" unbounded="{@unbounded}">
		<xsl:call-template name="produce-idref-element">
			<xsl:with-param name="name" select="@ref"/>
			<xsl:with-param name="attname">group</xsl:with-param>
		</xsl:call-template>
	</xsc:groupUse>
</xsl:template>


<xsl:template match="xsc:group">

	<xsc:group id="{generate-id(.)}">

		<xsl:variable name="childGroups" select="xsc:group"/>
		<xsl:variable name="anyOrElement" select="xsc:element|xsc:any"/>
		<xsl:variable name="test" select="count($childGroups)=1 and not($anyOrElement) and $childGroups/@minOccurs=1 and $childGroups/@maxOccurs=1 and $childGroups/@unbounded='false'"/>
		<!-- test described in model group definition   -->
		<xsl:choose>
			<xsl:when test="$test">
				<!--  determine compositor    -->
				<xsl:attribute name="compositor"><xsl:value-of select="$childGroups/@compositor"/></xsl:attribute>
				<xsl:apply-templates select="xsc:annotation"/>
				<xsl:apply-templates select="xsc:group/*" mode="element-use"/>
			</xsl:when>

			<xsl:otherwise>
				<xsc:group compositor="sequence" minOccurs="1" maxOccurs="1">
					<xsl:apply-templates select="xsc:annotation"/>
					<xsl:apply-templates select="xsc:group|xsc:element|xsc:any"/>
				</xsc:group>
			</xsl:otherwise>

		</xsl:choose>
	</xsc:group>
</xsl:template>

<xsl:template name="produce-idref">
	<xsl:param name="name"/>
	<xsl:param name="attname"/>
	<xsl:param name="keyname"/>
	<xsl:param name="default"/>
	<xsl:param name="suffix"/>
	<xsl:param name="fallbackNS"/>
	<xsl:param name="fallbackKey"/>

	<xsl:choose>
		<!--  when no name is specified find the default   -->
		<xsl:when test="string-length($name)=0">
			<xsl:attribute name="{$attname}">
				<xsl:value-of select="generate-id(key($keyname,$default))"/>
			</xsl:attribute>
		</xsl:when>


		<!-- if the name contains an explicit namespace prefix  -->
		<xsl:when test="contains($name,':')">
			<xsl:variable name="prefix" select="substring-before($name,':')"/>
			<xsl:variable name="namespace" select="/xsc:schema/xsc:namespace[xsc:prefix=$prefix]/@uri"/>
			<xsl:choose>
				<xsl:when test="$namespace">
					<xsl:variable name="tag" select="concat(concat($namespace,' '),substring-after($name,':'))"/>
					<xsl:variable name="target" select="key($keyname,$tag)"/>
					<xsl:choose>
						<xsl:when test="$target">
							<xsl:attribute name="{$attname}"><xsl:value-of select="concat(generate-id($target),$suffix)"/></xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:message terminate="yes">Fatal error: unable to resolve <xsl:value-of select="$name"/> in keytable <xsl:value-of select="$keyname"/>.</xsl:message>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>

				<xsl:otherwise>
					<xsl:message terminate="yes">Fatal error: unable to match prefix in <xsl:value-of select="$name"/> to a schema.  May need to add xsc:prefix element to the appinfo annotation of the schema.</xsl:message>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:when>

		<!--  unqualified name, try this schema first then schema for schemas   -->
		<xsl:otherwise>
			<xsl:variable name="defaultNS" select="ancestor::xsc:schema/@targetNamespace"/>
			<xsl:variable name="hereKey" select="concat(concat($defaultNS,' '),$name)"/>
			<xsl:variable name="targetHere" select="key($keyname,$hereKey)"/>
			<xsl:choose>
				<xsl:when test="$targetHere">
					<xsl:attribute name="{$attname}"><xsl:value-of select="concat(generate-id($targetHere),$suffix)"/></xsl:attribute>
				</xsl:when>

				<xsl:otherwise>
					<xsl:if test="$fallbackNS">
						<xsl:variable name="targetXSD" select="key($fallbackKey,concat(concat($fallbackNS,' '),$name))"/>
						<xsl:choose>
							<xsl:when test="$targetXSD">
								<xsl:attribute name="{$attname}"><xsl:value-of select="generate-id($targetXSD)"/></xsl:attribute>
							</xsl:when>
							<xsl:otherwise>
								<xsl:message terminate="yes">Fatal Error : Unable to resolve name <xsl:value-of select="$name"/>.</xsl:message>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:if>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>


<!--  it is ugly, but apparently you must pass in a literal as the first argument to the key() function 
          so I haven't found a way to parameterize this code    -->
<xsl:template name="produce-idref-simple">
	<xsl:param name="name"/>
	<xsl:param name="attname"/>

	<xsl:call-template name="produce-idref">
		<xsl:with-param name="name" select="$name"/>
		<xsl:with-param name="attname" select="$attname"/>
		<xsl:with-param name="keyname">named-simple-types</xsl:with-param>
		<xsl:with-param name="default">http://www.w3.org/1999/XMLSchema string</xsl:with-param>
		<xsl:with-param name="fallbackNS">http://www.w3.org/1999/XMLSchema</xsl:with-param>
		<xsl:with-param name="fallbackKey">named-simple-types</xsl:with-param>
	</xsl:call-template>
</xsl:template>

<xsl:template name="produce-idref-element">
	<xsl:param name="name"/>
	<xsl:param name="attname"/>

	<xsl:call-template name="produce-idref">
		<xsl:with-param name="name" select="$name"/>
		<xsl:with-param name="attname" select="$attname"/>
		<xsl:with-param name="keyname">named-elements</xsl:with-param>
	</xsl:call-template>
</xsl:template>


<xsl:template name="produce-idref-complex">
	<xsl:param name="name"/>
	<xsl:param name="attname"/>
	<xsl:param name="suffix"/>

	<xsl:call-template name="produce-idref">
		<xsl:with-param name="name" select="$name"/>
		<xsl:with-param name="attname" select="$attname"/>
		<xsl:with-param name="keyname">named-complex-types</xsl:with-param>
		<xsl:with-param name="suffix" select="$suffix"/>
	</xsl:call-template>
</xsl:template>


<xsl:template name="produce-idref-type">
	<xsl:param name="name"/>
	<xsl:param name="attname"/>

	<xsl:call-template name="produce-idref">
		<xsl:with-param name="name" select="$name"/>
		<xsl:with-param name="attname" select="$attname"/>
		<xsl:with-param name="keyname">named-types</xsl:with-param>
		<xsl:with-param name="default">http://www.w3.org/1999/XMLSchema string</xsl:with-param>
		<xsl:with-param name="fallbackNS">http://www.w3.org/1999/XMLSchema</xsl:with-param>
		<xsl:with-param name="fallbackKey">named-simple-types</xsl:with-param>
	</xsl:call-template>
</xsl:template>

<xsl:template name="produce-idref-attribute">
	<xsl:param name="name"/>
	<xsl:param name="attname"/>

	<xsl:call-template name="produce-idref">
		<xsl:with-param name="name" select="$name"/>
		<xsl:with-param name="attname" select="$attname"/>
		<xsl:with-param name="keyname">named-attributes</xsl:with-param>
	</xsl:call-template>
</xsl:template>


<xsl:template name="produce-simple-type">
	<xsl:attribute name="id"><xsl:value-of select="generate-id(.)"/></xsl:attribute>
	<xsl:call-template name="produce-idref-simple">
		<xsl:with-param name="name" select="@base"/>
		<xsl:with-param name="attname">base</xsl:with-param>
	</xsl:call-template>
	<xsl:if test="@derivedBy='list'">
		<xsl:attribute name="list">1</xsl:attribute>
	</xsl:if>
	<!--  copy in all facets -->
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="xsc:simpleType">
	<xsl:param name="namespaceID"/>
	<xsc:simpleType>
		<xsl:if test="@name">
			<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
			<xsl:attribute name="namespace"><xsl:value-of select="$namespaceID"/></xsl:attribute>

			<!--   check for duplicately named simple types   -->
			<xsl:variable name="nameCount" select="count(key('named-types',concat(concat(ancestor::xsc:schema/xsc:namespace/@uri,' '),@name)))"/>
			<xsl:if test="$nameCount != 1">
				<xsl:message terminate="yes">Fatal error: Duplicate types with name <xsl:value-of select="@name"/> in namespace <xsl:value-of select="ancestor::xsc:schema/xsc:namespace/@uri"/>.</xsl:message>
			</xsl:if>
		</xsl:if>
		<xsl:call-template name="produce-simple-type"/>
	</xsc:simpleType>
</xsl:template>



<xsl:template match="xsc:complexType">
	<xsc:complexType id="{generate-id(.)}" group="{generate-id(.)}_group" attributes="{generate-id(.)}_attr_group">
		<xsl:if test="@name">
			<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>

			<!--   check for duplicately named types   -->
			<xsl:variable name="nameCount" select="count(key('named-types',concat(concat(ancestor::xsc:schema/xsc:namespace/@uri,' '),@name)))"/>
			<xsl:if test="$nameCount != 1">
				<xsl:message terminate="yes">Fatal error: Duplicate types with name <xsl:value-of select="@name"/> in namespace <xsl:value-of select="ancestor::xsc:schema/xsc:namespace/@uri"/>.</xsl:message>
			</xsl:if>
		</xsl:if>

		<xsl:if test="@base">		
			<xsl:call-template name="produce-idref-complex">
				<xsl:with-param name="name" select="@base"/>
				<xsl:with-param name="keyname" select="named-complex-types"/>
				<xsl:with-param name="attname">base</xsl:with-param>
			</xsl:call-template>
		</xsl:if>

		<xsl:apply-templates select="@abstract"/>
		<xsl:apply-templates select="@content"/>
		<xsl:apply-templates select="@final-restriction"/>
		<xsl:apply-templates select="@final-extension"/>
		<xsl:apply-templates select="@block-restriction"/>
		<xsl:apply-templates select="@block-equivClass"/>

		<xsl:apply-templates select="xsc:annotation"/>

	</xsc:complexType>
</xsl:template>


<xsl:template match="xsc:notation">
	<xsc:notation>
		<xsl:apply-templates select="@*|xsc:annotation"/>
	</xsc:notation>
</xsl:template>

<xsl:template match="xsc:annotation">
	<xsc:annotation>
		<xsl:apply-templates select="xsc:appinfo|xsc:documentation"/>
	</xsc:annotation>
</xsl:template>

<xsl:template match="xsc:appinfo">
	<xsc:appinfo>
		<xsl:apply-templates select="@*|*|text()"/>
	</xsc:appinfo>
</xsl:template>

<xsl:template match="xsc:documentation">
	<xsc:documentation>
		<xsl:apply-templates select="@*|*|text()"/>
	</xsc:documentation>
</xsl:template>

<xsl:template match="xsc:attribute">
	<xsl:param name="namespaceID"/>

	<!--  if we are a top level attribute   -->
	<xsl:if test="parent::xsc:schema">
		<!--  there had better be only one of us    -->
		<xsl:variable name="duplicateCount" select="count(key('named-attributes',concat(concat(parent::xsc:schema/@targetNamespace,' '),@name)))"/>
		<xsl:if test="$duplicateCount!=1">
			<xsl:message terminate="yes">Duplicately named top level attribute <xsl:value-of select="@name"/>.</xsl:message>
		</xsl:if>
	</xsl:if>

	<xsc:attribute id="{generate-id(.)}" name="{@name}" form="{@form}" namespace="{$namespaceID}">

		<xsl:choose>
			<!--  if there is an anonymous type   -->
			<xsl:when test="xsc:simpleType">
				<xsl:attribute name="type"><xsl:value-of select="generate-id(xsc:simpleType)"/></xsl:attribute>
			</xsl:when>

			<xsl:otherwise>
				<xsl:call-template name="produce-idref-simple">
					<xsl:with-param name="name" select="@type"/>
					<xsl:with-param name="schema" select="ancestor::xsc:schema"/>
					<xsl:with-param name="keyname" select="named-simple-types"/>
					<xsl:with-param name="attname">type</xsl:with-param>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>

		<!--   process any annotation (removing any nested simple type)     -->
		<xsl:apply-templates select="xsc:annotation"/>
	</xsc:attribute>
</xsl:template>

<xsl:template match="xsc:attribute" mode="attribute-use">
	<xsc:attributeUse id="{generate-id(.)}_use" attribute="{generate-id(.)}">

		<xsl:apply-templates select="@use"/>
		<xsl:apply-templates select="@fixed"/>
		<xsl:apply-templates select="@default"/>

	</xsc:attributeUse>
</xsl:template>

<xsl:template match="xsc:element[@name]">
	<xsl:param name="namespaceID"/>


	<xsc:element id="{generate-id(.)}" name="{@name}" nullable="{@nullable}" abstract="{@abstract}" namespace="{$namespaceID}">

		<!--   if a namespace ID (top level element) was passed then add it to the attributes -->
		<xsl:if test="$namespaceID">
			<xsl:attribute name="namespace"><xsl:value-of select="$namespaceID"/></xsl:attribute>
		</xsl:if>

		<xsl:variable name="childType" select="xsc:simpleType|xsc:complexType"/>
		<xsl:choose>
			<xsl:when test="$childType">
				<xsl:attribute name="type"><xsl:value-of select="generate-id($childType)"/></xsl:attribute>
			</xsl:when>

			<xsl:when test="@type">
				<xsl:call-template name="produce-idref-type">
					<xsl:with-param name="name" select="@type"/>
					<xsl:with-param name="attname">type</xsl:with-param>
				</xsl:call-template>
			</xsl:when>

			<xsl:otherwise>
				<!--  content was empty    -->
			</xsl:otherwise>
		</xsl:choose>

		<!--  if there was an equivClass reference, then determine the id of the examplar    -->
		<xsl:if test="@equivClass">
			<xsl:call-template name="produce-idref-element">
				<xsl:with-param name="name" select="@equivClass"/>
				<xsl:with-param name="attname">equivClass</xsl:with-param>
			</xsl:call-template>
		</xsl:if>

		<!--  copy any annotation -->
		<xsl:apply-templates select="xsc:annotation"/>

	</xsc:element>
</xsl:template>

<xsl:template match="xsc:attribute" mode="attribute-uses">
	<xsc:attributeUse attribute="{generate-id(.)}">
		<xsl:apply-templates select="@default"/>
		<xsl:apply-templates select="@fixed"/>
		<xsl:apply-templates select="@use"/>
		<xsl:apply-templates select="xsc:annotation"/>
	</xsc:attributeUse>
</xsl:template>

<xsl:template match="xsc:attributeGroup[@name]" mode="attribute-uses">
	<!--  check for duplicate named attribute groups   -->
	<xsl:variable name="count-same-name" select="count(key('named-attr-groups',concat(concat(ancestor::xsc:schema/@targetNamespace,' '),@name)))"/>
	<xsl:if test="$count-same-name!=1">
		<xsl:message terminate="yes">Fatal Error: There are <xsl:value-of select="$count-same-name"/> attribute groups with the name <xsl:value-of select="@name"/> in namespace <xsl:value-of select="ancestor::xsc:schema/xsc:namespace/@uri"/>.</xsl:message>
	</xsl:if>

	<xsc:attributeGroup id="{generate-id(.)}">
		<xsl:apply-templates select="xsc:attributeGroup|xsc:attribute|xsc:annotation" mode="attribute-uses"/>
	</xsc:attributeGroup>
</xsl:template>

<xsl:template match="xsc:annotation" mode="attribute-uses">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="*" mode="attribute-uses">
	<xsl:message>Warning: Internal Error, this template should not get executed.  Tag name <xsl:value-of select="name()"/></xsl:message>
	<xsl:comment>Warning: Internal Error, this template should not get executed.  Tag name <xsl:value-of select="name()"/></xsl:comment>
</xsl:template>

<xsl:template match="xsc:namespace" mode="namespace-ref">
	<xsc:namespaceRef namespace="{generate-id(key('namespaceURI',@uri))}"/>
</xsl:template>

<xsl:template match="*" mode="namespace-ref">
	<xsl:message terminate="yes">Fatal Error: Internal Error, template should be unreached.  Element <xsl:value-of select="name()"/></xsl:message>
</xsl:template>

<xsl:template match="xsc:anyAttribute" mode="attribute-uses">
	<xsc:anyAttribute>
		<xsl:apply-templates select="xsc:annotation"/>
		<xsl:if test="xsc:namespaceNor">
			<xsc:namespaceNor>
				<xsl:apply-templates select="xsc:namespaceNor/*" mode="namespace-ref"/>
			</xsc:namespaceNor>
		</xsl:if>
		<xsl:apply-templates select="xsc:namespace" mode="namespace-ref"/>
	</xsc:anyAttribute>
</xsl:template>

<xsl:template match="xsc:attribute" mode="attribute-restriction">
	<xsc:attributeRestriction id="{generate-id(.)}">
		<!--  these attributes are unchanged, name wasn't being produced for some reason   -->
		<xsl:apply-templates select="@*[name()!='type' and name()!='ref']"/>

		<!-- if an type attribute is specified, this means that
		         the attribute value must conform to both the base type and this type  -->
		<xsl:if test="@type">
			<xsl:call-template name="produce-idref-simple">
				<xsl:with-param name="name" select="@type"/>
				<xsl:with-param name="attname">conform</xsl:with-param>
			</xsl:call-template>
		</xsl:if>

		<!--  if a simple type has a base, it could be a whole different animal
		              so make it the target of a conform    
		              -->
		<xsl:for-each select="xsc:simpleType[@base]">
			<xsl:attribute name="conform"><xsl:value-of select="generate-id(.)"/></xsl:attribute>
		</xsl:for-each>

		<!--  if it didn't have a base, then it is just a bag of facets
		         that need to be interpreted in terms of the ultimate type of the 
				 attribute.  Unfortunately, we will have to find that out next pass.   
				 -->
		<xsl:variable name="facets" select="xsc:simpleType[not(@base)]"/>
		<xsl:if test="$facets">
			<xsc:restrictionFacets>
				<xsl:apply-templates select="$facets"/>
			</xsc:restrictionFacets>
		</xsl:if>


		<!--  if this restriction refers to a top-level attribute
		            the resolve its id    -->
		<xsl:if test="@ref">
			<xsl:call-template name="produce-idref-attribute">
				<xsl:with-param name="name" select="@ref"/>
				<xsl:with-param name="attname">ref</xsl:with-param>
			</xsl:call-template>
		</xsl:if>

	</xsc:attributeRestriction>
</xsl:template>


<xsl:template match="xsc:complexType" mode="attribute-uses">
	<xsl:param name="namespaceID"/>

	<xsl:comment>Generate attribute group for complex type <xsl:value-of select="@name"/>.</xsl:comment>

	<xsl:variable name="base" select="@base"/>
	<xsc:attributeGroup id="{generate-id(.)}_attr_group">
		<xsl:apply-templates select="xsc:annotation"/>
		
		
		<xsl:for-each select="xsc:restriction">
			<xsl:for-each select="xsc:attribute">
				<xsc:attributeUseRestriction namespace="{$namespaceID}">
					<xsl:call-template name="produce-idref-complex">
						<xsl:with-param name="name" select="$base"/>
						<xsl:with-param name="keyname" select="named-complex-types"/>
						<xsl:with-param name="attname">base</xsl:with-param>
						<xsl:with-param name="suffix">_attr_group</xsl:with-param>
					</xsl:call-template>
					<xsl:apply-templates select="@*|*"/>
				</xsc:attributeUseRestriction>
			</xsl:for-each>

			<xsl:for-each select="xsc:anyAttribute">
				<xsc:anyAttributeRestriction>
					<xsl:call-template name="produce-idref-complex">
						<xsl:with-param name="name" select="@base"/>
						<xsl:with-param name="keyname" select="named-complex-types"/>
						<xsl:with-param name="attname">base</xsl:with-param>
						<xsl:with-param name="suffix">_attr_group</xsl:with-param>
					</xsl:call-template>
					<xsl:apply-templates/>
				</xsc:anyAttributeRestriction>
			</xsl:for-each>
		</xsl:for-each>

		<xsl:if test="@base">
			<xsc:attributeGroupRef>
				<xsl:call-template name="produce-idref-complex">
					<xsl:with-param name="name" select="@base"/>
					<xsl:with-param name="attname">ref</xsl:with-param>
					<xsl:with-param name="suffix">_attr_group</xsl:with-param>
				</xsl:call-template>
			</xsc:attributeGroupRef>
		</xsl:if>

		<xsl:apply-templates select="xsc:attributeGroup"/>

		<xsl:apply-templates select="xsc:attribute" mode="attribute-uses"/>

	</xsc:attributeGroup>
</xsl:template>


<xsl:template match="xsc:attributeGroup[@ref]" mode="attribute-uses">
	<xsl:choose>
		<!--  if the attribute group is a qualified name   -->
		<xsl:when test="contains(@ref,':')">

			<!--  KLUDGE: doing this evaluation in the namespace evaluation failed   -->
			<xsl:variable name="prefix" select="substring-before(@ref,':')"/>
			<xsl:variable name="namespace" select="ancestor::xsc:schema/xsc:namespace[xsc:prefix=$prefix]"/>
			<xsl:if test="count($namespace)!=1">
				<xsl:message terminate="yes">Fatal Error: Unable to resolve namespace prefix in <xsl:value-of select="@ref"/>.</xsl:message>
			</xsl:if>
			<xsl:variable name="target" select="key('named-attr-groups',concat(concat($namespace/@uri,' '),substring-after(@ref,':')))"/>
			<xsl:if test="count($target)!=1">
				<xsl:message terminate="yes">Fatal Error: Unable to resolve attribute group <xsl:value-of select="@ref"/> in namespace <xsl:value-of select="ancestor::xsc:schema/xsc:namespace/@uri"/>.</xsl:message>
			</xsl:if>

			<!--  expand attribute group     -->
			<xsc:attributeGroupRef idref="{generate-id($target)}"/>
		</xsl:when>

		<!--  if the attribute group is a not qualified name   -->
		<xsl:otherwise>
			<xsl:variable name="target" select="key('named-attr-groups',concat(concat(ancestor::xsc:schema/@targetNamespace,' '),@ref))"/>
			<xsl:if test="count($target)!=1">
				<xsl:message terminate="yes">Fatal Error: Unable to resolve attribute group <xsl:value-of select="@ref"/> in namespace <xsl:value-of select="ancestor::xsc:schema/xsc:namespace/@uri"/>.</xsl:message>
			</xsl:if>
			<!--  expand attribute group     -->
			<xsc:attributeGroupRef idref="{generate-id($target)}"/>
		</xsl:otherwise>
	</xsl:choose>	
</xsl:template>


</xsl:transform>